-------------------------------------------------------------------------------
-- Copyright (c) 2006 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
-------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 1.1
--  \   \        Filename: phy_top.vhd
--  /   /        Date Last Modified: 5/11/06
-- /___/   /\    Date Created:
-- \   \  /  \
--  \___\/\___\
-- 
--Device: Virtex-5
--Purpose: Top-level for memory "physical-layer" PHY interface. User would
--         instantiate this module (instead of either hwtb_ddr1_top or
--         ddr1_top) if they want to use their own memory controller, and only
--         need to use the physical layer logic (i.e. read data calibration and
--         capture, SDRAM initialization). Contains: (1) All SDRAM-related
--         IOBs, (2) SDRAM initialization logic, (3) write data path, (4) read
--         data path - specifically read data calibration/capture. Also
--         instantiated directly in this file (in addition to the major PHY
--         sub-modules): (1) I/O buffers for data/mask/DQS, (2) shift register
--         delay-chain for internal read enable valid signal.
--Reference:
--    XAPP851
--Revision History:
--    Rev 0.1 - Created. Author: Toshihiko Moriyama. 1/04/06.
--    Rev 1.0 - Internal release. Author: Toshihiko Moriyama. 4/29/06.
--    Rev 1.1 - External release. Added header. Added RST90 support.
--              Renamed OBUFT_DQ instance to OBUFT_DM. Changed CK to
--              vector. 5/11/06.
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_misc.all;

library unisim ;
use unisim.vcomponents.all ;

library work;
use work.ddr1_parameters.all;


entity phy_top is
port(

	rst				: in	std_logic;
        rst90                   : in    std_logic;
	clk0			: in	std_logic;
	clk90			: in	std_logic;

	-- IO signals (memory interface)
	CKE				: out	std_logic;
	CK				: out	std_logic_vector(clk_width - 1 downto 0);
	AD				: out	std_logic_vector(row_address - 1 downto 0);
	BA				: out	std_logic_vector(bank_address - 1 downto 0);
	CS_n			: out	std_logic_vector(no_of_cs - 1 downto 0);
	RAS_n			: out	std_logic;
	CAS_n			: out	std_logic;
	WE_n			: out	std_logic;
	DM				: out	std_logic_vector(data_mask_width - 1 downto 0);
	DQ				: inout	std_logic_vector(data_width - 1 downto 0);
	DQS				: inout	std_logic_vector(data_strobe_width - 1 downto 0);

	-- UI
	phy_rdy			: out	std_logic;
	phy_error		: out	std_logic;

	phy_addr_in		: in	std_logic_vector(row_address - 1 downto 0);
	phy_bank_in		: in	std_logic_vector(bank_address - 1 downto 0);
	phy_cs_n_in		: in	std_logic_vector(no_of_cs - 1 downto 0);
	phy_ras_n_in	: in	std_logic;
	phy_cas_n_in	: in	std_logic;
	phy_we_n_in		: in	std_logic;

	phy_wr_data_in	: in	std_logic_vector(data_width*2 - 1 downto 0);	-- Input write data with write command
	phy_wr_en_in	: in	std_logic;
	phy_wr_dm_in	: in	std_logic_vector(data_mask_width*2 - 1 downto 0);
	phy_rd_data_o	: out	std_logic_vector(data_width*2 - 1 downto 0);
	phy_rd_valid_o	: out	std_logic
);
end phy_top;

-----------------------------------------------
architecture rtl of phy_top is

	component phy_adr_out is
	port(
	rst		: in	std_logic;
	clk0	: in	std_logic;

	-- internal interface
	addr_in	: in	std_logic_vector(row_address - 1 downto 0);
	bank_in	: in	std_logic_vector(bank_address - 1 downto 0);

	-- Memory interface
	AD		: out	std_logic_vector(row_address - 1 downto 0);
	BA		: out	std_logic_vector(bank_address - 1 downto 0)
	);
	end component;

	component phy_ctrl_out is
	port(
	rst		: in	std_logic;
	clk0		: in	std_logic;

	-- internal interface
	cs_n_in		: in	std_logic_vector(no_of_cs - 1 downto 0);
	ras_n_in	: in	std_logic;
	cas_n_in	: in	std_logic;
	we_n_in		: in	std_logic;

	-- Memory interface
	CS_n		: out	std_logic_vector(no_of_cs - 1 downto 0);
	RAS_n		: out	std_logic;
	CAS_n		: out	std_logic;
	WE_n		: out	std_logic;
	CK		: out	std_logic_vector(clk_width - 1 downto 0)
	);
	end component;

	component phy_data_read is
	port(
	rst				: in	std_logic;
	clk0			: in	std_logic;

	-- internal interface
	rd_data			: out	std_logic_vector(dq_per_dqs*2 - 1 downto 0);
	rd_lat			: out	std_logic_vector(5 downto 0);
	latency			: in	std_logic_vector(3 downto 0);	-- Fixed latency
	rd_en_in		: in	std_logic;	-- from controller

	calib_start0	: in	std_logic;	-- calibration phase 1 starts
	calib_start1	: in	std_logic;	-- calibration phase 2 starts
	calib_done0		: out	std_logic;	-- calibration phase 1 done
	calib_done1		: out	std_logic;	-- calibration phase 2 done
	error			: out	std_logic;

	-- Memory interface
	DQ				: in	std_logic_vector(dq_per_dqs - 1 downto 0);
	DQS				: in	std_logic
	);
	end component;

	component phy_data_write is
	port(
	rst			: in	std_logic;
        rst90           : in    std_logic;
	clk0		: in	std_logic;
	clk90		: in	std_logic;

	-- internal interface
	wr_data		: in	std_logic_vector(dq_per_dqs*2 - 1 downto 0);
	wr_dm		: in	std_logic_vector(1 downto 0);
	wr_en		: in	std_logic;

	-- Memory interface
	DQ			: out	std_logic_vector(dq_per_dqs - 1 downto 0);
	DQ_T		: out	std_logic_vector(dq_per_dqs - 1 downto 0);
	DQS			: out	std_logic_vector(0 downto 0);
	DQS_T		: out	std_logic_vector(0 downto 0);
	DM			: out	std_logic_vector(0 downto 0);
	DM_T		: out	std_logic_vector(0 downto 0)
	);
	end component;

	component phy_init is
	port(
	rst				: in	std_logic;
	clk0			: in	std_logic;

	-- from/to memory
	-- During initialization, PHY controller issues commands
	cke_o			: out	std_logic;
	cs_n_o			: out	std_logic;
	ras_n_o			: out	std_logic;
	cas_n_o			: out	std_logic;
	we_n_o			: out	std_logic;
	addr_o			: out	std_logic_vector(row_address - 1 downto 0);
	bank_o			: out	std_logic_vector(bank_address - 1 downto 0);

	-- internal interface
	init_done		: out	std_logic
	);
	end component;

	component phy_ptn_gen is
	port(
	rst				: in	std_logic;
	clk0			: in	std_logic;

	-- Handshaking signals with controller
	init_done		: in	std_logic;
	calib_start0	: out	std_logic;	-- calib_start
	calib_start1	: out	std_logic;
	calib_done0		: in	std_logic;
	calib_done1		: in	std_logic;
	rd_rdy			: out	std_logic;

	cs_n_o			: out	std_logic;
	ras_n_o			: out	std_logic;
	cas_n_o			: out	std_logic;
	we_n_o			: out	std_logic;
	addr_o			: out	std_logic_vector(row_address - 1 downto 0);
	bank_o			: out	std_logic_vector(bank_address - 1 downto 0);
	wr_data_o		: out	std_logic_vector(data_width*2 - 1 downto 0);
	wr_en_o			: out	std_logic
	);
	end component;

	-- States
	TYPE DDR1_STATE_TYPE is (
	WAIT_INIT_DONE_ST,
	WAIT_CALIB_DONE_ST,
	MAIN_ST
	);

	signal state_c		: DDR1_STATE_TYPE;
	signal state_n		: DDR1_STATE_TYPE;

	signal addr_i		: std_logic_vector(row_address - 1 downto 0);
	signal bank_i		: std_logic_vector(bank_address - 1 downto 0);

	signal cke_i		: std_logic;
	signal cs_n_i		: std_logic_vector(no_of_cs - 1 downto 0);
	signal ras_n_i		: std_logic;
	signal cas_n_i		: std_logic;
	signal we_n_i		: std_logic;

	signal rd_latency	: std_logic_vector(3 downto 0);
	signal rd_en_in_i	: std_logic;
	signal rd_en_brst	: std_logic_vector(3 downto 0);

	signal wr_data_i	: std_logic_vector(data_width*2 - 1 downto 0);
	signal wr_dm_i		: std_logic_vector(data_mask_width*2 - 1 downto 0);
	signal wr_en_i		: std_logic;

	signal calib_start0		: std_logic;
	signal calib_start1		: std_logic;
	signal calib_done0		: std_logic;
	signal calib_done1		: std_logic;
	signal calib_done0_i	: std_logic_vector(no_of_comps*dqs_per_comp - 1 downto 0);
	signal calib_done1_i	: std_logic_vector(no_of_comps*dqs_per_comp - 1 downto 0);
	signal error_i			: std_logic_vector(no_of_comps*dqs_per_comp - 1 downto 0);
	signal rd_rdy			: std_logic;

	type slv6Array is array (no_of_comps*dqs_per_comp - 1 downto 0) of std_logic_vector(5 downto 0);
	signal rd_lat			: slv6Array;
	signal rd_lat8			: std_logic_vector(no_of_comps*dqs_per_comp - 1 downto 0);
	signal rd_lat7			: std_logic_vector(no_of_comps*dqs_per_comp - 1 downto 0);
	signal rd_lat6			: std_logic_vector(no_of_comps*dqs_per_comp - 1 downto 0);
	signal rd_lat5			: std_logic_vector(no_of_comps*dqs_per_comp - 1 downto 0);
	signal rd_lat4			: std_logic_vector(no_of_comps*dqs_per_comp - 1 downto 0);
	signal rd_lat3			: std_logic_vector(no_of_comps*dqs_per_comp - 1 downto 0);

	signal addr_init		: std_logic_vector(row_address - 1 downto 0);
	signal bank_init		: std_logic_vector(bank_address - 1 downto 0);
	signal cke_init			: std_logic;
	signal cs_n_init		: std_logic;
	signal ras_n_init		: std_logic;
	signal cas_n_init		: std_logic;
	signal we_n_init		: std_logic;
	signal init_done_i		: std_logic;

	signal addr_calib		: std_logic_vector(row_address - 1 downto 0);
	signal bank_calib		: std_logic_vector(bank_address - 1 downto 0);
	signal cs_n_calib		: std_logic;
	signal ras_n_calib		: std_logic;
	signal cas_n_calib		: std_logic;
	signal we_n_calib		: std_logic;
	signal wr_data_calib	: std_logic_vector(data_width*2 - 1 downto 0);
	signal wr_en_calib		: std_logic;

	signal phy_rd_data		: std_logic_vector(data_width*2 - 1 downto 0);
	signal phy_wr_data		: std_logic_vector(data_width*2 - 1 downto 0);
	signal phy_wr_dm		: std_logic_vector(data_mask_width*2 - 1 downto 0);

	signal DQ_in			: std_logic_vector(dq_per_comp * no_of_comps - 1 downto 0);
	signal DQ_o				: std_logic_vector(dq_per_comp * no_of_comps - 1 downto 0);
	signal DQ_T_o			: std_logic_vector(dq_per_comp * no_of_comps - 1 downto 0);
	signal DQS_in			: std_logic_vector(dqs_per_comp * no_of_comps - 1 downto 0);
	signal DQS_o			: std_logic_vector(dqs_per_comp * no_of_comps - 1 downto 0);
	signal DQS_T_o			: std_logic_vector(dqs_per_comp * no_of_comps - 1 downto 0);
	signal DM_o				: std_logic_vector(dm_per_comp * no_of_comps - 1 downto 0);
	signal DM_T_o			: std_logic_vector(dm_per_comp * no_of_comps - 1 downto 0);

	signal rd_en_A			: std_logic_vector(3 downto 0);
	signal rd_en_srl		: std_logic;
	signal phy_rdy_i		: std_logic;

	signal burst_length		: std_logic_vector(2 downto 0);
	signal command			: std_logic_vector(3 downto 0);	-- CS_n & RAS_n & CAS_n & WE_n
	-- commands used in init phase
	constant DESEL_CMD		: std_logic_vector(3 downto 0) := "1111";
	constant NOP_CMD		: std_logic_vector(3 downto 0) := "0111";
	constant BSTP_CMD		: std_logic_vector(3 downto 0) := "0110";
	constant PRECHARGE_CMD	: std_logic_vector(3 downto 0) := "0010";
	constant AUTOREF_CMD	: std_logic_vector(3 downto 0) := "0001";
	constant LMR_CMD		: std_logic_vector(3 downto 0) := "0000";
	constant ACT_CMD		: std_logic_vector(3 downto 0) := "0011";
	constant RD_CMD			: std_logic_vector(3 downto 0) := "0101";	-- not auto precharge
	constant WR_CMD			: std_logic_vector(3 downto 0) := "0100";	-- not auto precharge

begin

	burst_length <= load_mode_register(2 downto 0);

	G_DQ : for I in 0 to no_of_comps * dq_per_comp - 1 generate

	IOBUF_DQ : IOBUF
	port map (
	IO => DQ(I), O=> DQ_in(I), I => DQ_o(I), T => DQ_T_o(I)
	);

	end generate;

	G_DQS : for I in 0 to no_of_comps * dqs_per_comp - 1 generate

	IOBUF_DQS : IOBUF
	port map (
	IO => DQS(I), O => DQS_in(I), I => DQS_o(I), T => DQS_T_o(I)
	);

	end generate;

	G_DM : for I in 0 to no_of_comps * dm_per_comp - 1 generate

	OBUFT_DM : OBUFT
	port map (
	O => DM(I), I => DM_o(I), T => DM_T_o(I)
	);
	end generate;

	CKE <= cke_i;
	phy_rdy <= phy_rdy_i;

	-------------------------------------------------------
	--	DDR1 memory controller initialization state machine
	-------------------------------------------------------
	P_SM_NEXT : process(clk0)
	begin
	if clk0'event and clk0='1' then
		if rst = '1' then
			state_c <= WAIT_INIT_DONE_ST;
		else
			state_c <= state_n;
		end if;
	end if;
	end process;

	P_SM : process ( rst, state_c, init_done_i, rd_rdy )
	begin
	if rst='1' then
		state_n <= WAIT_INIT_DONE_ST;
	else
		case state_c is
		when WAIT_INIT_DONE_ST =>
			if init_done_i = '1' then
				state_n <= WAIT_CALIB_DONE_ST;
			else
				state_n <= WAIT_INIT_DONE_ST;
			end if;

		when WAIT_CALIB_DONE_ST =>
			if rd_rdy = '1' then
				state_n <= MAIN_ST;
			else
				state_n <= WAIT_CALIB_DONE_ST;
			end if;

		when MAIN_ST =>
			state_n <= MAIN_ST;

		when others =>
			state_n <= WAIT_INIT_DONE_ST;

		end case;
	end if;
	end process;

	P_OUT : process (clk0 )
	begin
	if clk0'event and clk0='1' then
		if rst='1' then
			cke_i		<= '0';
			cs_n_i		<= (others => '1');
			ras_n_i		<= '1';
			cas_n_i		<= '1';
			we_n_i		<= '1';
			addr_i		<= (others => '0');
			bank_i		<= (others => '0');
			wr_data_i	<= (others => '0');
			wr_dm_i		<= (others => '0');
			wr_en_i		<= '0';
			phy_rdy_i	<= '0';

		else
			case state_c is
			when WAIT_INIT_DONE_ST =>
				cke_i		<= cke_init;
				cs_n_i		<= (others => cs_n_init);
				ras_n_i		<= ras_n_init;
				cas_n_i		<= cas_n_init;
				we_n_i		<= we_n_init;
				addr_i		<= addr_init;
				bank_i		<= bank_init;
				wr_data_i	<= (others => '0');
				wr_dm_i		<= (others => '0');
				wr_en_i		<= '0';
				phy_rdy_i	<= '0';

			when WAIT_CALIB_DONE_ST =>
				cke_i		<= '1';
				cs_n_i		<= (others => cs_n_calib);
				ras_n_i		<= ras_n_calib;
				cas_n_i		<= cas_n_calib;
				we_n_i		<= we_n_calib;
				addr_i		<= addr_calib;
				bank_i		<= bank_calib;
				wr_data_i	<= wr_data_calib;
				wr_dm_i		<= (others => '0');
				wr_en_i		<= wr_en_calib;
				phy_rdy_i	<= '0';

			when MAIN_ST =>
				cke_i		<= '1';
				cs_n_i		<= phy_cs_n_in;
				ras_n_i		<= phy_ras_n_in;
				cas_n_i		<= phy_cas_n_in;
				we_n_i		<= phy_we_n_in;
				addr_i		<= phy_addr_in;
				bank_i		<= phy_bank_in;
				wr_data_i	<= phy_wr_data;
				wr_dm_i		<= phy_wr_dm;
				wr_en_i		<= phy_wr_en_in;
				phy_rdy_i	<= '1';

			when others =>
				cke_i		<= '0';
				cs_n_i		<= (others => '1');
				ras_n_i		<= '1';
				cas_n_i		<= '1';
				we_n_i		<= '1';
				addr_i		<= (others => '0');
				bank_i		<= (others => '0');
				wr_data_i	<= (others => '0');
				wr_dm_i		<= (others => '0');
				wr_en_i		<= '0';
				phy_rdy_i	<= '0';

			end case;
		end if;
	end if;
	end process;

	G_WR_ORDER : for I in 0 to no_of_comps * dqs_per_comp - 1 generate

		phy_wr_data(dq_per_dqs*2*(I+1)-1 downto dq_per_dqs*2*I) <=
							phy_wr_data_in(dq_per_dqs*(I+1)+dq_per_comp*no_of_comps-1 downto dq_per_dqs*I+dq_per_comp*no_of_comps) &
							phy_wr_data_in(dq_per_dqs*(I+1)-1 downto dq_per_dqs*I);

		phy_wr_dm(I*2+1 downto I*2) <= phy_wr_dm_in(I+dm_per_comp*no_of_comps) & phy_wr_dm_in(I);

	end generate;

	command <= AND_REDUCE(cs_n_i) & ras_n_i & cas_n_i & we_n_i;

	P_RD_EN : process(clk0)
	begin
	if clk0'event and clk0='1' then
		if rst='1' then
			rd_en_in_i <= '0';
			rd_en_brst <= (others => '0');
		else
			if command = RD_CMD then	-- if read
				case burst_length is
				when "001" =>
					rd_en_brst <= "1000";
				when "010" =>
					rd_en_brst <= "1100";
				when "011" =>
					rd_en_brst <= "1111";
				when others =>
					rd_en_brst <= rd_en_brst(2 downto 0) & '0';
				end case;
			elsif command = BSTP_CMD then	-- if read burst stop
				rd_en_brst <= (others => '0');
			else
				rd_en_brst <= rd_en_brst(2 downto 0) & '0';
			end if;
			rd_en_in_i <= rd_en_brst(3);
		end if;
	end if;
	end process;
	----------------------------------------------------------------

	PHY_ADR_OUT_I : phy_adr_out
	port map (
	rst			=> rst,
	clk0		=> clk0,

	-- internal interface
	addr_in		=> addr_i,
	bank_in		=> bank_i,

	-- Memory interface
	AD			=> AD,
	BA			=> BA
	);

	-- Control signals
	PHY_CTRL_OUT_I : phy_ctrl_out
	port map (
	rst			=> rst,
	clk0		=> clk0,

	-- internal interface
	cs_n_in		=> cs_n_i,
	ras_n_in	=> ras_n_i,
	cas_n_in	=> cas_n_i,
	we_n_in		=> we_n_i,

	-- Memory interface
	CS_n		=> CS_n,
	RAS_n		=> RAS_n,
	CAS_n		=> CAS_n,
	WE_n		=> WE_n,
	CK			=> CK
	);



	-- read data
	G_RD : for I in 0 to no_of_comps * dqs_per_comp - 1 generate

	PHY_DATA_READ_I : phy_data_read
	port map(
	rst				=> rst,
	clk0			=> clk0,

	-- internal interface
	rd_data			=> phy_rd_data(dq_per_dqs*2*(I+1) - 1 downto dq_per_dqs*2*I),
	rd_lat			=> rd_lat(I),
	latency			=> rd_latency,
	rd_en_in		=> rd_en_in_i,

	calib_start0	=> calib_start0,
	calib_start1	=> calib_start1,
	calib_done0		=> calib_done0_i(I),
	calib_done1		=> calib_done1_i(I),
	error			=> error_i(I),

	-- Memory interface
	DQ				=> DQ_in(dq_per_dqs*(I+1) - 1 downto dq_per_dqs*I),
	DQS				=> DQS_in(I)
	);

	rd_lat8(I) <= rd_lat(I)(5);
	rd_lat7(I) <= rd_lat(I)(4);
	rd_lat6(I) <= rd_lat(I)(3);
	rd_lat5(I) <= rd_lat(I)(2);
	rd_lat4(I) <= rd_lat(I)(1);
	rd_lat3(I) <= rd_lat(I)(0);

	-- rising edge group
	phy_rd_data_o(dq_per_dqs*(I+1)-1 downto dq_per_dqs*I)
							<= phy_rd_data(dq_per_dqs+dq_per_dqs*2*I-1 downto dq_per_dqs*2*I);
	-- falling edge group
	phy_rd_data_o(dq_per_dqs*(I+1)+dq_per_comp*no_of_comps-1 downto dq_per_dqs*I+dq_per_comp*no_of_comps)
							<= phy_rd_data(dq_per_dqs*2*(I+1)-1 downto dq_per_dqs+dq_per_dqs*2*I);


	end generate;

	-- pick max latency
	P_READ_LATENCY : process( clk0 )
	begin
	if clk0'event and clk0='1' then
		if rst='1' then
			rd_latency <= (others => '0');
		else
			if or_reduce(rd_lat8) = '1' then
				rd_latency <= "1000";
			elsif or_reduce(rd_lat7) = '1' then
				rd_latency <= "0111";
			elsif or_reduce(rd_lat6) = '1' then
				rd_latency <= "0110";
			elsif or_reduce(rd_lat5) = '1' then
				rd_latency <= "0101";
			elsif or_reduce(rd_lat4) = '1' then
				rd_latency <= "0100";
			elsif or_reduce(rd_lat3) = '1' then
				rd_latency <= "0011";
			else
				rd_latency <= "0000";
			end if;
		end if;
	end if;
	end process;

	rd_en_A <= unsigned(rd_latency) - CONV_UNSIGNED(1,4);
	phy_rd_valid_o <= rd_en_srl and phy_rdy_i;

	SRL16_RDEN : SRL16
	generic map (
	INIT => X"0000")
	port map (
	Q	=> rd_en_srl,
	A0	=> rd_en_A(0),
	A1	=> rd_en_A(1),
	A2	=> rd_en_A(2),
	A3	=> rd_en_A(3),
	CLK	=> clk0,
	D	=> rd_en_in_i
	);

	calib_done0 <= and_reduce(calib_done0_i);
	calib_done1 <= and_reduce(calib_done1_i);
	phy_error	<= or_reduce(error_i);

	-- Write IOB
	G_WR : for I in 0 to no_of_comps * dqs_per_comp - 1 generate

	PHY_DATA_WRITE_I : phy_data_write
	port map(
	rst			=> rst,
        rst90           => rst90,
	clk0		=> clk0,
	clk90		=> clk90,

	-- internal interface
	wr_data		=> wr_data_i(dq_per_dqs*2*(I+1) - 1 downto dq_per_dqs*2*I ),
	wr_dm		=> wr_dm_i(2*I+1 downto 2*I),
	wr_en		=> wr_en_i,

	-- Memory interface
	DQ			=> DQ_o(dq_per_dqs*(I+1) - 1 downto  dq_per_dqs*I),
	DQ_T		=> DQ_T_o(dq_per_dqs*(I+1) - 1 downto dq_per_dqs*I),
	DQS			=> DQS_o(I downto I),
	DQS_T		=> DQS_T_o(I downto I),
	DM			=> DM_o(I downto I),
	DM_T		=> DM_T_o(I downto I)
	);

	end generate;


	PHY_INIT_I : phy_init
	port map (
	rst				=> rst,
	clk0			=> clk0,

	-- To IOB
	cke_o			=> cke_init,
	cs_n_o			=> cs_n_init,
	ras_n_o			=> ras_n_init,
	cas_n_o			=> cas_n_init,
	we_n_o			=> we_n_init,
	addr_o			=> addr_init,
	bank_o			=> bank_init,

	init_done		=> init_done_i
	);


	PHY_PTN_GEN_I : phy_ptn_gen
	port map (
	rst				=> rst,
	clk0			=> clk0,

	-- Handshaking signals with controller
	init_done		=> init_done_i,
	calib_start0	=> calib_start0,
	calib_start1	=> calib_start1,
	calib_done0		=> calib_done0,
	calib_done1		=> calib_done1,
	rd_rdy			=> rd_rdy,

	cs_n_o			=> cs_n_calib,
	ras_n_o			=> ras_n_calib,
	cas_n_o			=> cas_n_calib,
	we_n_o			=> we_n_calib,
	addr_o			=> addr_calib,
	bank_o			=> bank_calib,
	wr_data_o		=> wr_data_calib,
	wr_en_o			=> wr_en_calib
	);

end rtl;
